//
//  ViewController.swift
//  StampCamera
//
//  Created by IKKO OGAWA on 2016/11/20.
//  Copyright © 2016年 Nenriki. All rights reserved.
//

import UIKit

class ViewController: UIViewController {

    @IBOutlet weak var imageView: UIImageView!
    
    private var stampImage = UIImage(named: "heart")
    
    // MARK: - Life cycle
    override func viewDidLoad() {
        super.viewDidLoad()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    // MARK: - View controller
    override func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?) {
        
        guard let point = touches.first?.location(in: imageView) else { return }
        let stampSize: CGFloat = 32
        let maxY = imageView.frame.maxY - stampSize
        
        guard maxY > point.y else { return }
        
        addStamp(with: point)
    }

    // MARK: - IBAction
    @IBAction func takePictureTapped(_ sender: UIButton) {
        
        let sourceType = UIImagePickerControllerSourceType.camera
        
        if UIImagePickerController.isSourceTypeAvailable(.camera) {
            let cameraPicker = UIImagePickerController()
            cameraPicker.sourceType = sourceType
            cameraPicker.delegate = self
            
            present(cameraPicker, animated: true, completion: nil)
        } else {
            displayAlert(title: "エラー", message: "カメラが有効になっていません")
        }
    }
    
    @IBAction func saveButtonTapped(_ sender: UIButton) {
        saveImageWithStamps()
    }
    
    @IBAction func undoButtonTapped(_ sender: UIButton) {
        let stamps = imageView.subviews
        
        stamps.last?.removeFromSuperview()
    }
    
    @IBAction func clearButtonTapped(_ sender: UIButton) {
        let stamps = imageView.subviews
        
        stamps.forEach { subview in
            subview.removeFromSuperview()
        }
    }
    
    @IBAction func firstStampButtonTapped(_ sender: UIButton) {
        stampImage = UIImage(named: "heart")
    }
    
    @IBAction func secondStampButtonTapped(_ sender: UIButton) {
        stampImage = UIImage(named: "star")
    }
    
    @IBAction func thirdStampButtonTapped(_ sender: UIButton) {
        stampImage = UIImage(named: "snow")
    }
    
    // MARK: - Private
    private func displayAlert(title: String, message: String) {
        let alertController = UIAlertController(title: title, message: message, preferredStyle: .alert)
        let action = UIAlertAction(title: "OK", style: .cancel, handler: nil)
        
        alertController.addAction(action)
        
        present(alertController, animated: true, completion: nil)
    }
    
    private func addStamp(with point: CGPoint) {
        
        let stampSize = CGSize(width: 32, height: 32)
        let imageView = UIImageView(frame: CGRect(origin: CGPoint.zero, size: stampSize))
        
        imageView.center = point
        
        imageView.image = stampImage
        
        self.imageView.addSubview(imageView)
    }
    
    private func saveImageWithStamps() {
        
        UIGraphicsBeginImageContext(imageView.bounds.size)
        
        guard let context = UIGraphicsGetCurrentContext() else {
            displayAlert(title: "エラー", message: "画像の保存に失敗しました")
            return
        }
        
        imageView.layer.render(in: context)
        
        guard let image = UIGraphicsGetImageFromCurrentImageContext() else {
            displayAlert(title: "エラー", message: "画像の保存に失敗しました")
            return
        }
        
        UIImageWriteToSavedPhotosAlbum(image, self, #selector(ViewController.image(image:didFinishSaveImageWithError:contextInfo:)), nil)
        
        UIGraphicsEndImageContext()
    }
    
    // MARK: - Internal
    func image(image: UIImage, didFinishSaveImageWithError error: NSError?, contextInfo: UnsafeMutableRawPointer) {
        
        switch error {
        case let .some(error):
            displayAlert(title: "エラー", message: error.localizedDescription)
            
        case .none:
            displayAlert(title: "成功", message: "画像が保存されました！")
        }
    }
}

// MARK: - Image picker controller delegate
extension ViewController: UIImagePickerControllerDelegate {
    
    func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [String : Any]) {
        if let takedImage = info[UIImagePickerControllerOriginalImage] as? UIImage {
            imageView.image = takedImage
        }
        
        picker.dismiss(animated: true, completion: nil)
    }
    
    func imagePickerControllerDidCancel(_ picker: UIImagePickerController) {
        picker.dismiss(animated: true, completion: nil)
    }
}

// MARK: - Navigation controller delegate
extension ViewController: UINavigationControllerDelegate {}


